#!/bin/bash -u

# setup args in the right order for making getopt evaluation
# nice and easy.  You'll need to read the manpages for more info
args=$(getopt -o h -- "$@")
eval set -- "$args"

###########################################################
usage() {
    echo ''           >&2
    echo "$0 <%post number>" >&2
    echo "   This script is really just an easy to call" >&2
    echo "   wrapper around authconfig" >&2
    exit 1
}

###########################################################
for arg in $@; do
    case $1 in
        -- )
            # end of getopt args, shift off the -- and get out of the loop
            shift
            break 2
           ;;
         -h )
            # get help
            usage
           ;;

    esac
done

###########################################################

CURRENT_INSTALLS_OF_THIS_PACKAGE=${1:-0}

NOW=$(date +%x_%T)
TMPFILE=$(mktemp)

which augtool >/dev/null 2>&1
if [[ $? -ne 0 ]]; then
    echo "Could not find augtool" >&2
    exit 1
fi

source /etc/os-release

if [[ ${CURRENT_INSTALLS_OF_THIS_PACKAGE} -eq 0 ]]; then
    # only remove the config on removing this rpm, not upgrading
    logger -p auth.info 'Removing fermilab-conf_sssd'
    touch /etc/sssd/sssd.conf
    chmod 600 /etc/sssd/sssd.conf
    restorecon -Rf /etc/sssd/ >/dev/null 2>&1
    grep -q '^\[sssd\]' /etc/sssd/sssd.conf
    if [[ $? -ne 0 ]]; then
        echo "[sssd]" >> /etc/sssd/sssd.conf
    fi

    cat > ${TMPFILE} <<EOFA
match /files/etc/sssd/sssd.conf/*[. = 'sssd']/domains
EOFA
    MATCHES=$(augtool <${TMPFILE})
    if [[ $? -ne 0 ]]; then
        exit 1
    fi

    echo ${MATCHES} | grep -q 'FNAL'
    if [[ $? -eq 0 ]]; then
        # remove domain from list
        DOMAINS=$(echo ${MATCHES} | cut -d= -f2)
        if [[ "${DOMAINS}" != 'FNAL' ]]; then
            DOMAINS=$(echo ${DOMAINS} | sed -e 's/FNAL//')
            DOMAINS=$(echo ${DOMAINS} | sed -e 's/,,//')
            DOMAINS=$(echo ${DOMAINS} | sed -e 's/ *$//')
            DOMAINS=$(echo ${DOMAINS} | sed -e 's/,*$//')
            cat > ${TMPFILE} <<EOFB
set /files/etc/sssd/sssd.conf/*[. = 'sssd']/domains ${DOMAINS}
EOFB
            OUT=$(augtool -s <${TMPFILE})
            if [[ $? -ne 0 ]]; then
                exit 1
            fi
            systemctl restart sssd > /dev/null 2>&1
        else
            cat > ${TMPFILE} <<EOFC
rm /files/etc/sssd/sssd.conf/*[. = 'sssd']/domains
EOFC
            OUT=$(augtool -s <${TMPFILE})
            systemctl disable sssd > /dev/null 2>&1
            systemctl stop sssd > /dev/null 2>&1
        fi
    fi

else
    logger -p auth.info 'Installing fermilab-conf_sssd'

    touch /etc/sssd/sssd.conf
    chmod 600 /etc/sssd/sssd.conf
    restorecon -Rf /etc/sssd/ >/dev/null 2>&1
    grep -q '^\[sssd\]' /etc/sssd/sssd.conf
    if [[ $? -ne 0 ]]; then
        echo "[sssd]" >> /etc/sssd/sssd.conf
    fi

    cat > ${TMPFILE} <<EOFD
match /files/etc/sssd/sssd.conf/*[. = 'sssd']/domains
EOFD
    MATCHES=$(augtool <${TMPFILE})
    if [[ $? -ne 0 ]]; then
        exit 1
    fi

    echo ${MATCHES} | grep -qe '(no matches)\|(none)'
    if [[ $? -eq 0 ]]; then
        cat > ${TMPFILE} <<EOFE
set /files/etc/sssd/sssd.conf/*[. = 'sssd']/domains FNAL
EOFE
        OUT=$(augtool -s <${TMPFILE})
        if [[ $? -ne 0 ]]; then
            exit 1
        fi
    else
        echo ${MATCHES} | grep -q 'FNAL'
        if [[ $? -ne 0 ]]; then
            DOMAINS=$(echo ${MATCHES} | cut -d= -f2)
            cat > ${TMPFILE} <<EOFF
set /files/etc/sssd/sssd.conf/*[. = 'sssd']/domains ${DOMAINS}, FNAL
EOFF
            OUT=$(augtool -s <${TMPFILE})
            if [[ $? -ne 0 ]]; then
                exit 1
            fi

        fi
    fi

    cat > ${TMPFILE} <<EOFG
match /files/etc/sssd/sssd.conf/*[. = 'sssd']/services
EOFG
    MATCHES=$(augtool <${TMPFILE})
    if [[ $? -ne 0 ]]; then
        exit 1
    fi

    echo ${MATCHES} | grep -qe '(no matches)\|(none)'
    if [[ $? -eq 0 ]]; then
        cat > ${TMPFILE} <<EOFH
set /files/etc/sssd/sssd.conf/*[. = 'sssd']/services 'nss pam'
EOFH
        OUT=$(augtool -s <${TMPFILE})
        if [[ $? -ne 0 ]]; then
            exit 1
        fi
    else
        SERVICES=$(echo ${MATCHES} | cut -d= -f2)
        SERVICES="${SERVICES} pam nss"
        SERVICES=$(echo ${SERVICES} | tr ' ' '\012' | sort -u)
        cat > ${TMPFILE} <<EOFI
set /files/etc/sssd/sssd.conf/*[. = 'sssd']/services '${SERVICES}'
EOFI
        OUT=$(augtool -s <${TMPFILE})
        if [[ $? -ne 0 ]]; then
            exit 1
        fi
    fi


    which authselect >/dev/null 2>&1
    if [[ $? -eq 0 ]]; then
        OUT=$(authselect select sssd)
        logger -p auth.info 'ran authselect select sssd'
    else
        which authconfig >/dev/null 2>&1
        if [[ $? -eq 0 ]]; then
            OUT=$(authconfig --update --enablesssdauth --savebackup=${NOW}_fermilab-conf_sssd)
            logger -p auth.info 'ran authconfig --update --enablesssdauth'
        fi
    fi

    systemctl enable sssd-nss.socket sssd-pam.socket >/dev/null 2>&1
    systemctl start sssd-nss.socket sssd-pam.socket >/dev/null 2>&1
    systemctl enable sssd >/dev/null 2>&1
    systemctl restart sssd >/dev/null 2>&1

fi
